"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getConfig = exports.flagStack = exports.isFlagOrSynonymInCommandLine = exports.makeFlagWithPrefix = exports.getConfigValueByFlagOrSynonym = exports.parsedCLIParams = exports.flagAliases = exports.flagsWithDefaulValues = exports.withFlagCredentials = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const btoa_1 = __importDefault(require("btoa"));
const atob_1 = __importDefault(require("atob"));
const yargs_parser_1 = __importDefault(require("yargs-parser"));
const printMessage_1 = require("./printMessage");
const tsUtils_1 = require("./tsUtils");
const parseURL_1 = require("./parseURL");
exports.withFlagCredentials = false;
exports.flagsWithDefaulValues = {
    'dry-run': false,
    'save-dev': false,
    'save-prod': false,
    'types-only': false,
    'ignore-package-json': false,
    'allow-prerelease': false,
    silent: false,
    registry: '',
    _auth: '',
    login: '',
    password: '',
    completion: false,
    version: false,
    help: false,
    ':make-autocompletion-flags': false,
    ':test-flags-run': false,
    // 'check-package-json-only': false,
    // 'ts-version': '',
};
exports.flagAliases = {
    'dry-run': 'd',
    'save-dev': 'D',
    'save-prod': 'P',
    'types-only': 't',
    'ignore-package-json': 'i',
    'allow-prerelease': 'ap',
    silent: 's',
    registry: 'r',
    _auth: 'a',
    login: ['l', 'username', 'u'],
    password: 'p',
    completion: 'c',
    version: 'v',
    help: 'h',
    ':make-autocompletion-flags': [],
    ':test-flags-run': [],
    // 'check-package-json-only': 'co',
};
function isFlag(e) {
    return tsUtils_1.ObjKeys(exports.flagsWithDefaulValues).includes(e);
}
function transformOptions(opts) {
    const tOptions = {
        boolean: [],
        string: [],
        alias: {},
    };
    const allFlagSynonyms = [];
    for (const [flag, val] of tsUtils_1.ObjEntries(opts)) {
        const flagSynonyms = Array.isArray(val) ? val : [val];
        const flagType = typeof exports.flagsWithDefaulValues[flag];
        if (flagType === 'boolean' || flagType === 'string') {
            tOptions[flagType].push(flag);
        }
        else {
            printMessage_1.printMessage.warning(`Attention: type of flag "${flag}" is neither string nor boolean!`);
        }
        tOptions.alias[flag] = flagSynonyms;
        for (const flagSynonym of flagSynonyms) {
            if (!allFlagSynonyms.includes(flagSynonym)) {
                allFlagSynonyms.push(flagSynonym);
            }
            else {
                printMessage_1.printMessage.warning(`Attention: synonym "${flagSynonym}" of flag "${flag}" is a duplicate!`);
            }
        }
    }
    return tOptions;
}
const parserOptions = {
    ...transformOptions(exports.flagAliases),
    configuration: {
        'camel-case-expansion': false,
        'dot-notation': false,
        'parse-positional-numbers': false,
        'boolean-negation': false,
        'duplicate-arguments-array': false,
        'strip-aliased': true,
        'unknown-options-as-args': true,
        'negation-prefix': '',
    },
};
function parseCLIParams() {
    const cliParams = process.argv.slice(2).join(' ');
    const parsedParams = yargs_parser_1.default(cliParams, parserOptions);
    const argv = {
        ...exports.flagsWithDefaulValues,
        ...Object.fromEntries(tsUtils_1.ObjEntries(parsedParams)
            .map(e => {
            const [key, val] = e;
            if (val === false) {
                parsedParams._.push('false');
                return [key, true];
            }
            return e;
        })),
    };
    argv[':test-flags-run'] = false;
    if (!argv[':make-autocompletion-flags']) {
        const unknownFlags = argv._.filter(e => e.startsWith('-'));
        if (unknownFlags.length) {
            printMessage_1.printMessage.error(`Unknown flag "${unknownFlags[0]}".`);
            process.exit(1);
        }
    }
    argv._ = [...new Set(argv._)];
    if (argv.registry || argv._auth || argv.login || argv.password) {
        exports.withFlagCredentials = true;
    }
    try {
        const { registry, _auth, login, password } = Object.fromEntries(fs_1.default
            .readFileSync(path_1.default.resolve('.npmrc'))
            .toString()
            .trim()
            .split('\n')
            .map(row => (row
            .split('=')
            .map(e => e.trim()))));
        if (!argv.registry && registry) {
            argv.registry = registry;
        }
        if (!argv._auth && _auth) {
            argv._auth = _auth;
        }
        if (!argv.login && login) {
            argv.login = login;
        }
        if (!argv.password && password) {
            argv.password = password;
        }
    }
    catch { }
    if (!argv.registry) {
        argv.registry = 'https://registry.npmjs.org';
    }
    try {
        const { registry } = argv;
        const { hrefWithNoCredentials, username, password } = parseURL_1.parseURL(registry);
        argv.registry = hrefWithNoCredentials;
        if (username) {
            argv.login = username;
        }
        if (password) {
            argv.password = password;
        }
        if (argv.login && argv.password) {
            argv._auth = btoa_1.default(`${argv.login}:${argv.password}`);
        }
        if (argv._auth) {
            const [login, password] = atob_1.default(argv._auth).split(':');
            if (login) {
                argv.login = login;
            }
            if (password) {
                argv.password = password;
            }
        }
    }
    catch { }
    return argv;
}
exports.parsedCLIParams = parseCLIParams();
function getConfigValueByFlagOrSynonym(key) {
    if (isFlag(key)) {
        return exports.parsedCLIParams[key];
    }
    const alias = tsUtils_1.ObjEntries(exports.flagAliases).find(([, e]) => e === key || e.includes(key));
    if (!alias) {
        throw new Error('This should never happen.');
    }
    return exports.parsedCLIParams[alias[0]];
}
exports.getConfigValueByFlagOrSynonym = getConfigValueByFlagOrSynonym;
const clearHyphens = (flag) => flag.replace(/^-+/, '');
const clearPostfixEqualitySigns = (flag) => flag.replace(/=+$/, '');
const clearFlag = (flag) => clearHyphens(clearPostfixEqualitySigns(flag));
const makeFlagWithPrefix = (flag) => clearFlag(flag).length > 1 ? `--${flag}` : `-${flag}`;
exports.makeFlagWithPrefix = makeFlagWithPrefix;
function isFlagOrSynonymInCommandLine(key) {
    const cliParams = process.argv.slice(2).join(' ');
    const { aliases } = yargs_parser_1.default.detailed(cliParams, parserOptions);
    const keys = [
        ...aliases[clearFlag(key)].map(exports.makeFlagWithPrefix),
        clearPostfixEqualitySigns(key),
    ];
    const arr = cliParams
        .split(/\s+/)
        .map(e => e.split(/=/)[0])
        .flatMap(e => {
        const match = e.match(/^-([a-z]+)$/i);
        return !match ? e : match[1].split('').map(e => `-${e}`);
    });
    return arr.some(e => keys.includes(e));
}
exports.isFlagOrSynonymInCommandLine = isFlagOrSynonymInCommandLine;
exports.flagStack = {};
function getConfig(flag) {
    exports.flagStack[flag] = true;
    return exports.parsedCLIParams[flag];
}
exports.getConfig = getConfig;
if (typeof exports.default === 'function') {
    module.exports = exports.default;
    Object.assign(module.exports, exports);
}
