"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.printMessage = void 0;
const parsedCLIParams_1 = require("./parsedCLIParams");
const command_line_usage_1 = __importDefault(require("command-line-usage"));
const parsedCLIParams_2 = require("./parsedCLIParams");
const tsUtils_1 = require("./tsUtils");
const chalk_1 = __importDefault(require("chalk"));
const pathToPackageJson = '../../package.json';
function multilineTrimLeft(strings, ...params) {
    var _a;
    const s = strings.map((e, i) => `${e}${i === params.length ? '' : params[i]}`).join('');
    const whiteSpaceAmount = (_a = s.match(/\n?(\s*)\S/)) === null || _a === void 0 ? void 0 : _a[1].length;
    let result = s.trim();
    if (whiteSpaceAmount) {
        const re = new RegExp(`\\n[\\t ]{1,${whiteSpaceAmount}}`, 'g');
        result = result.replace(re, '\n');
    }
    return result;
}
const noTypesPackages = [];
exports.printMessage = {
    noTypesDeclaration(packageName) {
        if (parsedCLIParams_1.getConfig('silent') || parsedCLIParams_1.getConfig(':test-flags-run')) {
            return;
        }
        noTypesPackages.push(packageName);
        if (noTypesPackages.length === 2) {
            this.warning('Notice that the next packages have no type declarations:');
            noTypesPackages.forEach(e => this.warning(` - ${e}`));
        }
        if (noTypesPackages.length > 2) {
            this.warning(` - ${packageName}`);
        }
    },
    endNoTypesDeclaration() {
        if (parsedCLIParams_1.getConfig('silent') || parsedCLIParams_1.getConfig(':test-flags-run')) {
            return;
        }
        if (noTypesPackages.length === 1) {
            this.warning(`Notice that "${noTypesPackages[0]}" has no type declarations.`);
        }
        if (noTypesPackages.length) {
            console.warn();
        }
    },
    noPackage(packageName) {
        if (parsedCLIParams_1.getConfig('silent') || parsedCLIParams_1.getConfig(':test-flags-run')) {
            return;
        }
        this.endNoTypesDeclaration();
        this.error(`No such package as ${packageName}.`);
    },
    noParams() {
        if (parsedCLIParams_1.getConfig('silent') || parsedCLIParams_1.getConfig(':test-flags-run')) {
            return;
        }
        this.error('No packages to install. Provide some params.');
    },
    unauthorized() {
        if (parsedCLIParams_1.getConfig('silent') || parsedCLIParams_1.getConfig(':test-flags-run')) {
            return;
        }
        this.error('Unauthorized. Provide correct _auth Basic token or pair login and password.');
    },
    version() {
        if (parsedCLIParams_1.getConfig(':test-flags-run')) {
            return;
        }
        const { version } = require(pathToPackageJson);
        console.info(`v${version}`);
    },
    help() {
        if (parsedCLIParams_1.getConfig(':test-flags-run')) {
            return;
        }
        const { name, version } = require(pathToPackageJson);
        const optionDescriptions = {
            'dry-run': 'Run without installing any packages.',
            'save-dev': 'Save regular (not {italic @types}) packages to devDependencies.',
            'save-prod': 'Save {italic @types} packages to regular dependencies.',
            'types-only': 'Install only {italic @types} packages.',
            'ignore-package-json': 'Add "latest" tag for all installing packages in case of latest versions are required to install.',
            'allow-prerelease': 'Allow to install prerelease versions of {italic @types} packages.',
            silent: 'Hide output messages.',
            registry: 'Specify any other package registry besides https://registry.npmjs.org (this value can be stored in .npmrc as {bold registry} key).',
            _auth: 'Set Auth Basic {italic token} for custom registry (this value can be stored in .npmrc as {bold _auth} key).',
            login: 'Set Auth Basic {italic username} for custom registry (this value can be stored in .npmrc as {bold login} key).',
            password: 'Set Auth Basic {italic password} for custom registry (this value can be stored in .npmrc as {bold password} key).',
            completion: 'Output bash CLI autocompletion alias and exit.',
            version: 'Output version information and exit.',
            help: 'Display this help and exit.',
            ':make-autocompletion-flags': '',
            ':test-flags-run': '',
        };
        const options = tsUtils_1.ObjEntries(optionDescriptions)
            .filter(e => e[1])
            .flatMap(([name, desc]) => {
            const synonyms = parsedCLIParams_2.flagAliases[name];
            const aliases = Array.isArray(synonyms) ? synonyms : [synonyms];
            const alias = aliases.shift();
            const type = { name: typeof parsedCLIParams_2.flagsWithDefaulValues[name] };
            const description = (() => {
                const [firstWord, ...restWords] = desc.split(' ');
                return `{underline ${firstWord}} ${restWords.join(' ')}`;
            })();
            const result = [{
                    name,
                    description,
                    alias: alias && parsedCLIParams_1.makeFlagWithPrefix(alias).slice(1),
                    type,
                }];
            while (aliases.length) {
                const synonymName = aliases.shift();
                const alias = aliases.shift();
                result.push({
                    name: synonymName,
                    alias,
                    description: `{underline Alias} for {bold ${name}}.`,
                    type,
                });
            }
            return result;
        });
        console.info(command_line_usage_1.default([
            {
                header: 'NAME',
                content: `${name} — add packages with their corresponding types. Version ${version}.`,
            },
            {
                header: 'ALIASES',
                content: multilineTrimLeft `
          {bold tsa} — for {bold ${name}}
          {bold tsi} — for {bold ${name} -i}
        `,
            },
            {
                header: 'SYNOPSIS',
                content: (() => {
                    const [completion, version, help,] = [
                        'completion',
                        'version',
                        'help',
                    ];
                    const [c, v, h] = ['c', 'v', 'h'];
                    return multilineTrimLeft `
            ${name} [{bold -${c}} {underline ${completion}}] [{bold -${v}} {underline ${version}}] [{bold -${h}} {underline ${help}}]
            ${name} [options]... package[@version] [[options] [package[@version]]]...
          `;
                })(),
            },
            {
                header: 'DESCRIPTION',
                content: `Some of NPM packages provide explicit types, some — external (as @types packages), rest — don't have any. ${name} helps to find and install NPM packages together with their corresponding types.`,
            },
            {
                header: 'EXAMPLES',
                content: (() => {
                    const [completion, dryRun, registry,] = [
                        'completion',
                        'dry-run',
                        'registry',
                    ];
                    const [i, P, D, r] = ['i', 'P', 'D', 'r'];
                    return multilineTrimLeft `
            ${name} --${completion}
            tsa node-fetch@^2.0 axios decline-word@latest --${dryRun}
            ${name} -${i}${P}${D} decline-word elapsing-time
            tsi my-awesome-module -${r}=https://my-awesome-${registry}.com
            tsa -t node@${process.version.slice(1)}
          `;
                })(),
            },
            {
                header: 'OPTIONS',
                optionList: options,
            },
            {
                header: 'EXIT STATUS',
                content: multilineTrimLeft `
          0   if OK,
          1   if any problems.
        `,
            },
            {
                header: 'AUTHOR',
                content: 'Designed and developed by ArthurKa.',
            },
            {
                header: 'ISSUES AND BUG REPORTING',
                content: `Your improve suggestions and bug reports are welcome any time at https://github.com/ArthurKa/${name}/issues.`,
            },
            {
                header: 'SEE ALSO',
                content: multilineTrimLeft `
          For more information visit:
           - NPM (https://www.npmjs.com/package/${name}/v/${version})
           - GitHub (https://github.com/ArthurKa/${name})
        `,
            },
        ]));
    },
    completion() {
        if (parsedCLIParams_1.getConfig(':test-flags-run')) {
            return;
        }
        const { name } = require(pathToPackageJson);
        console.info(multilineTrimLeft `
      ###-begin-${name}-completion-###
      # ${name} command completion script
      # Installation: ${name} --completion >> ~/.bashrc
      #
      if type complete &>/dev/null; then
        _ts_add_completion() {
          local cur word_number
          _get_comp_words_by_ref -n : cur
          word_number=$(grep -o ":" <<< "$COMP_LINE" | wc -l)

          COMPREPLY=($(compgen -W '$($1 --:make-autocompletion-flags\\
            \${COMP_WORDS[COMP_CWORD]} "\${COMP_LINE}")' -- "$cur"))
        }

        complete -F _ts_add_completion ${name}
        complete -F _ts_add_completion tsa
        complete -F _ts_add_completion tsi
      fi
      ###-end-${name}-completion-###
    `);
    },
    command(command) {
        if (parsedCLIParams_1.getConfig('silent')) {
            return;
        }
        console.info(chalk_1.default.bold(command));
    },
    error(command) {
        console.error(chalk_1.default.red(command));
    },
    warning(command) {
        console.warn(chalk_1.default.yellow(command));
    },
};
if (typeof exports.default === 'function') {
    module.exports = exports.default;
    Object.assign(module.exports, exports);
}
